#!/usr/bin/python3
import sys
from PyQt5 import QtCore, QtWidgets
import openBSTGUI
import bstController

class MainWindow(QtWidgets.QMainWindow, openBSTGUI.Ui_OpenBST):

    def __init__(self, parent = None):
        super(MainWindow, self).__init__(parent)
        self.setupUi(self)
        self.BST = bstController.BSTController()
        self.update()
        # PIL
        self.OpenPIL.clicked.connect(self.BST.PILShutter.OPEN)
        self.ClosePIL.clicked.connect(self.BST.PILShutter.CLOSE)
        # LINAC
        self.OpenLINACthr = Executor(self.BST.LinacStopper.OPEN)
        self.OpenLINAC.clicked.connect(self.OpenLINACthr.start)
        self.CloseLINACthr = Executor(self.BST.LinacStopper.CLOSE)
        self.CloseLINAC.clicked.connect(self.CloseLINACthr.start)
        # FEL1
        self.OpenFEL1thr = Executor(self.BST.openFEL1)
        self.OpenFEL1.clicked.connect(self.OpenFEL1thr.start)
        self.CloseFEL1thr = Executor(self.BST.closeFEL1)
        self.CloseFEL1.clicked.connect(self.CloseFEL1thr.start)
        # FEL2
        self.OpenFEL2thr = Executor(self.BST.openFEL2)
        self.OpenFEL2.clicked.connect(self.OpenFEL2thr.start)
        self.CloseFEL2thr = Executor(self.BST.closeFEL2)
        self.CloseFEL2.clicked.connect(self.CloseFEL2thr.start)
        # Trigger
        #self.PILTRG.clicked.connect(self.BST.PILTrigger.RESET)
        self.SLTRG.clicked.connect(self.BST.SEEDTrigger.RESET)
        
        #k2 trigger shift
        #self.checkBoxTriggerShift.clicked.connect(self.BST.trigShiftChanged)
        self.pushButtonTShiftEnable.clicked.connect(self.BST.trigShiftEnable)
        self.pushButtonTShiftDisable.clicked.connect(self.BST.trigShiftDisable)


        # collegamenti del menu
        self.action_menu_Silent_SEED.triggered.connect(self.BST.SEEDTrigger.SHIFT)
        self.action_menu_FEL1_shutter.triggered.connect(self.BST.FEL1Shutter.CLOSE)
        self.action_menu_FEL2_shutter.triggered.connect(self.BST.FEL2Shutter.CLOSE)
        # Timer for GUI update
        self.timer = QtCore.QBasicTimer()
        self.timer.start(1000, self)

    def __del__(self):
        self.timer.stop()

    def timerEvent(self, event):
        self.update()

    def update(self):
        PILShutter = self.BST.PILShutter.State()
        LINACStopper = self.BST.LinacStopper.State()
        FEL1 = self.stateForFEL(self.BST.FEL1Stopper.State(), self.BST.FEL1Shutter.State())
        FEL2 = self.stateForFEL(self.BST.FEL2Stopper.State(), self.BST.FEL2Shutter.State())
        #SEEDTrigger = self.BST.SEEDTrigger.State()
        SEEDTrigger = 'Open'
        opMode = self.BST.FERMImode.State()
        timeToGo = self.BST.TimeToGo.value
        triggerShift = self.BST.TriggerShift.value
        # print(triggerShift)
        #update trigger shift
        self.checkBoxTriggerShift.setChecked(triggerShift)

        # PIL shutter status
        self.setStateStringEnableColor(self.StatusPIL, PILShutter, changeString=True)
        # LINAC status
        self.setStateStringEnableColor(self.StatusLINAC, LINACStopper, changeString=True)
        # FEL status
        self.setStateStringEnableColor(self.StatusFEL1, FEL1, changeString=True)
        self.setStateStringEnableColor(self.StatusFEL2, FEL2, changeString=True)
        # Trigger Status
        self.setStateStringEnableColor(self.SLTRG, self.boolToState(SEEDTrigger), enabled=(not SEEDTrigger))
            
        # Enabled buttons
        if opMode == 'FEL01':
            self.OpenPIL.setEnabled(True)
            self.OpenLINAC.setEnabled(True)
            self.OpenFEL1.setEnabled(True)
            self.OpenFEL2.setEnabled(False)
        elif opMode == 'FEL02':
            self.OpenPIL.setEnabled(True)
            self.OpenLINAC.setEnabled(True)
            self.OpenFEL1.setEnabled(False)
            self.OpenFEL2.setEnabled(True)         
        else:
            self.OpenPIL.setEnabled(True)
            self.OpenLINAC.setEnabled(False)
            self.OpenFEL1.setEnabled(False)
            self.OpenFEL2.setEnabled(False)
        
        # menu shutter
        if opMode == 'FEL01':
            self.menuClose_Shutter.setEnabled(True)
            self.action_menu_FEL1_shutter.setEnabled(True)
            self.action_menu_FEL2_shutter.setEnabled(False)
        elif opMode == 'FEL02':
            self.menuClose_Shutter.setEnabled(True)
            self.action_menu_FEL1_shutter.setEnabled(False)
            self.action_menu_FEL2_shutter.setEnabled(True)
        else:
            self.menuClose_Shutter.setEnabled(False)

        # menu SEED trigger    
        if SEEDTrigger:
            self.action_menu_Silent_SEED.setEnabled(True)
        else:
            self.action_menu_Silent_SEED.setEnabled(False)
            
        # update time to go        
        self.timeToGo.setPlainText(timeToGo)

        # update fermi status - beamto main indicator
        self.setBooleanColor(self.label_gun, self.BST.fstatus.getAttrValue('beamto_gun'))
        self.setBooleanColor(self.label_linac, self.BST.fstatus.getAttrValue('beamto_linac'))
        self.label_opmode.setText(opMode)
        if opMode == 'FEL01':
            self.setStringBooleanColor(self.label_fel1, self.BST.fstatus.getAttrValue('beamto_fel1'),'FEL 1')
        elif opMode == 'FEL02':
            self.setStringBooleanColor(self.label_fel1, self.BST.fstatus.getAttrValue('beamto_fel2'), 'FEL 2')
        else:
            self.setStringBooleanColor(self.label_fel1, False, 'FEL')

        self.setBooleanColor(self.label_PADRES, self.BST.fstatus.getAttrValue('beamto_padres'))
        #self.setBooleanColor(self.label_lines, self.BST.fstatus.getAttrValue('beamto_lines'))

        # update fermi status - beamto detail indicators
        self.setStringBooleanColor(self.mscr_gun, self.BST.fstatus.getAttrValue('z_internal_gun_multiscreen'),'')
        self.setStringBooleanColor(self.vlv_gun, self.BST.fstatus.getAttrValue('z_internal_gun_valves'), '')
        self.setStringBooleanColor(self.mscr_linac, self.BST.fstatus.getAttrValue('z_internal_linac_multiscreen'), '')
        self.setStringBooleanColor(self.vlv_linac, self.BST.fstatus.getAttrValue('z_internal_linac_valves'), '')

        if opMode== 'FEL01':
            self.setStringBooleanColor(self.mscr_fel, self.BST.fstatus.getAttrValue('z_internal_fel1_multiscreen'), '')
            self.setStringBooleanColor(self.vlv_fel, self.BST.fstatus.getAttrValue('z_internal_fel1_valves'), '')
            self.setStringBooleanColor(self.bst_fel, self.BST.fstatus.getAttrValue('z_internal_fel1_bst'), '')
            self.setStringBooleanColor(self.vlv_padres, self.BST.fstatus.getAttrValue('z_internal_padres1_valves'), '')
            self.setStringBooleanColor(self.shut_padres, self.BST.fstatus.getAttrValue('z_internal_padres1_shutter'), '')

        elif opMode == 'FEL02':
            self.setStringBooleanColor(self.vlv_fel, self.BST.fstatus.getAttrValue('z_internal_fel2_valves'), '')
            self.setStringBooleanColor(self.bst_fel, self.BST.fstatus.getAttrValue('z_internal_fel2_bst'), '')
            self.setStringBooleanColor(self.vlv_padres, self.BST.fstatus.getAttrValue('z_internal_padres2_valves'), '')
            self.setStringBooleanColor(self.shut_padres, self.BST.fstatus.getAttrValue('z_internal_padres2_shutter'), '')
            #TODO : opMode == BC01
        else:
            self.setStringBooleanColor(self.mscr_fel, False, '---')
            self.setStringBooleanColor(self.vlv_fel, False, '---')
            self.setStringBooleanColor(self.bst_fel, False, '---')
            self.setStringBooleanColor(self.vlv_padres, False, '---')
            self.setStringBooleanColor(self.shut_padres, False, '---')

        #update terafermistatus
        instru = self.BST.terafermistatus.getAttrValue('Instrument')
        if instru == 'OffBeam':
            self.setStringBooleanColor(self.label_tf,True,'TeraFermi ON')
        else:
            self.setStringBooleanColor(self.label_tf,False,'TeraFermi OFF')











    @staticmethod
    def stateForFEL(stateStopper, stateShutter):
        if stateStopper == 'CLOSED':
            return 'CLOSED'
        elif stateShutter == 'CLOSED' or stateShutter == 'CLOSE':
            return 'SHUTTER'
        else:
            return 'OPEN'
        
    @staticmethod
    def boolToState(status):
        if status:
            return 'OPEN'
        else:
            return 'CLOSED'
        
    @staticmethod
    def setStateStringEnableColor(guiElement, state, changeString=False, enabled=True):
        if state == 'OPEN' or state == 'ON':
            guiElement.setStyleSheet("background-color: #00ff00")
        elif state == 'CLOSE' or state == 'CLOSED':
            guiElement.setStyleSheet("background-color: #ff0000")
        elif state == 'SHUTTER' or state == 'STANDBY':
            guiElement.setStyleSheet("background-color: yellow")
        else:
            guiElement.setStyleSheet("background-color: None")
        if changeString:
            guiElement.setText(state)
        guiElement.setEnabled(enabled)

    @staticmethod
    def setBooleanColor(guiElement, value, is_valid=True):
        if is_valid:
            if value:
                guiElement.setStyleSheet("background-color: #00ff00")
            else:
                guiElement.setStyleSheet("background-color: #ff0000")
        else:
            guiElement.setStyleSheet("background-color: None")

    @staticmethod
    def setStringBooleanColor(guiElement, value, message ,is_valid=True):
        if is_valid:
            guiElement.setText(message)
            if value:
                guiElement.setStyleSheet("background-color: #00ff00")
            else:
                guiElement.setStyleSheet("background-color: #ff0000")
        else:
            guiElement.setText('##')
            guiElement.setStyleSheet("background-color: None")

class Executor(QtCore.QThread):
    def __init__(self, funct, parent = None):
        super(Executor, self).__init__(parent)
        self.function = funct
        
    def run(self):
        # print("Running")
        self.function()

app = QtWidgets.QApplication(sys.argv)
main = MainWindow()
main.show()
sys.exit(app.exec_())
